# Procedury i funkcje pomocnicze dla przykładów Python
# Matematyka Dla Ciekawych Świata edycja XV Bis
# Krzysztof Lasocki <krz.lasocki@gmail.com>
# Licencja MIT
# SPDX-License-Identifier: MIT
# Wersja 1.1
# 1.1: Fix lambdy czas aby móc generować niecałkowite liczby sekund

import numpy as np
import matplotlib.pyplot as plt
import matplotlib.pyplot as plt
import numpy as np
import math
import librosa
from IPython.lib.display import Audio
from scipy.io import wavfile
import warnings

# scipy.io.wavfile.read wypisuje warningi gdy czyta WAV'y z tagami ID3
warnings.simplefilter("ignore")


czas = lambda t, fs: np.linspace(0, float(t), int(t*fs)) 


def fourier(x: np.ndarray, fs=16000, **kwargs):
    if type(x) != list:
        x = [x]
    fig, ax = plt.subplots(figsize=(7, 5))
    for sygnal in x:
        homega = np.fft.fftshift(np.fft.fft(sygnal))
        homf = np.fft.fftshift(np.fft.fftfreq(len(sygnal))) * fs
        ax.plot(homf, abs(homega), **kwargs)
    ax.grid()
    ax.set_xlabel("Częstotliwość")
    ax.set_ylabel("Wartość")
    ax.set_xlim(0, fs//2)
    return (fig, ax)

def charakterystyka(x, fs=16000, **kwargs):
    return fourier(np.concatenate((x, np.zeros(fs-len(x)))), fs=fs, **kwargs)

def spektrogram_muz(x: np.ndarray, fs=16000, tytul=None, **kwargs):
    D = librosa.stft(x)  # STFT of y
    S_db = librosa.amplitude_to_db(np.abs(D), ref=np.max)
    fig, ax = plt.subplots(figsize=(12, 4))
    img = librosa.display.specshow(S_db, sr=fs, x_axis='time', y_axis='fft_note', ax=ax, **kwargs)
    if tytul:
        ax.set(title=tytul)
    fig.colorbar(img, ax=ax, format="%+2.f dB")

def spektrogram(x: np.ndarray, fs=16000, tytul=None, **kwargs):
    D = librosa.stft(x)  # STFT of y
    S_db = librosa.amplitude_to_db(np.abs(D), ref=np.max)
    fig, ax = plt.subplots(figsize=(12, 4))
    img = librosa.display.specshow(S_db, sr=fs, x_axis='time', y_axis='linear', ax=ax, **kwargs)
    if tytul:
        ax.set(title=tytul)
    fig.colorbar(img, ax=ax, format="%+2.f dB")

# Wczytaj WAV z miksowaniem do mono
def wczytaj_wav(plik: str):
    fs, data = wavfile.read(plik)
    syg = data[:,0] * 0.5 + data[:,1] * 0.5
    return (syg, fs)

def rysuj_filtr(h):
    fig, ax = plt.subplots(figsize=(7, 5))
    ax.plot(h, "kx")
    ax.grid()
    ax.set_ylim(-0.3,1)

def wsp_norm(h):
    hns = 0
    for k in h:
        hns = hns + k
    return hns
